% Inspect the integration point quantities.
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
% Input arguments
%    x=reference geometry field
%    u=displacement field
%    dT=temperature difference field
%     context    - structure: see the update() method.
%     gcell_list - indexes of the geometric cells that are to be inspected
%     inspector - function handle or in-line function with the signature
%             inspector_data =inspector(inspector_data, out, xyz),
%        where
%     inspector_data - a structure or an array that the inspector may
%                use to maintain some state, for instance minimum or
%                maximum of stress, out is the output  of the update()
%                method, and xyz is the location of the integration point
%                in the *reference* configuration.
% Output arguments
%     inspector_data - see the description of the input argument
%
function inspector_data = inspect_integration_points(self, x, u, dT, gcell_list, context, inspector, inspector_data)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(x,'dim');
% Material
mat = get(self.feblock, 'mater');
matstates = get(self.feblock, 'matstates');
context.gradU= [];
% Make the inverse map from finite element nodes to geometric cells
f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
gmap=get (f2gmap,'gcell_map');
fenodemap=get (f2gmap,'fenodes');
% Now loop over all gcells in the block
for m=1:length(gcell_list)
    i=gcell_list(m);
    iconn = get(gcells(i), 'conn'); % connectivity
    for node=1:length(iconn)
        nix =iconn(node);
        gl=gmap{nix};
        thisnn=fenodemap{nix};
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            lnmap =0*(1:length(gmap));
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                        lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            % now store the local numbers
            for k=1:length(patchconn)
                lnmap(patchconn(k)) =k;
            end
            lnmap(thisnn) =length(patchconn)+1;
            % material tangent
            D = tangent_moduli (mat, struct ('ms',matstates{nix,1}));
            c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
            % preallocate the strain-displacement matrix
            es=local_basis(self,[]); % calculate the local Cartesian basis at the node
            % preallocate the strain-displacement matrix
            Nspatialderavg=zeros((length(patchconn)+1),2);
            Apatch =0;
            tpatch =0;
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
                lx(:,1)=lx(:,1)-c(1);
                lx(:,2)=lx(:,2)-c(2);
                Nder = Ndermat_param (gcells(gl(k)), [1 1]/3);
                xl=lx*es(:,1:2);% local coordinates
                [Nspatialder,detF] = Ndermat_spatial (gcells(gl(k)), Nder, xl);
                Ae = detF * 1/2*1/3;% were taking one third of the triangle
                Apatch=Apatch+Ae;
                tpatch=tpatch+Ae*thickness(gcells(gl(k)), [1 1]/3);
                Nspatialderavg(lnmap(kconn),:)=Nspatialderavg(lnmap(kconn),:)+Ae*Nspatialder;
            end
            tpatch=tpatch/Apatch;
            Nspatialderavg=Nspatialderavg/Apatch;
            conn= [patchconn thisnn];
            X = gather(x, conn, 'values', 'noreshape'); % reference coordinates
            U = gather(u, conn, 'values', 'noreshape'); % displacement
            if isempty(dT)
                dTs=zeros(length(conn),1);
            else
                dTs = gather(context.dT, conn, 'values', 'noreshape'); % displacement
            end
            context.gradU = def_grad(gcells(i), Nspatialderavg, U);
            context.dT = dTs(end); % temperature at the node
            [out,ignore] = update(mat, matstates{nix,1}, context);
            xyz = X(end,:);% location of the node
            if ~isempty (inspector)
                inspector_data =feval(inspector,inspector_data,out,xyz);
            end
        end
    end
end
return;
