% Compute the thermal-strain load vectors of the individual biaxial gcells.
% Return an array of the element vectors so they may be assembled.
%    Call as
% ems = thermal_strain_loads(feb, geom, u, T)
%     geom=geometry field
%     u=displacement field
%     dT=temperature difference field (temperature at a point minus the
%     reference temperature at which the solid experiences no strains)
%
function evs = thermal_strain_loads(self, geom, u, dT)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(geom,'dim');
% Pre-allocate the element matrices
evs(1:ngcells) = deal(elevec);
% Integration rule
integration_rule = get(self.feblock, 'integration_rule');
pc = get(integration_rule, 'param_coords');
w  = get(integration_rule, 'weights');
npts_per_gcell = get(integration_rule, 'npts');
% Material
mat = get(self.feblock, 'mater');
matstates = get(self.feblock, 'matstates');
I2=eye (dim,dim);
% Now loop over all gcells in the block
for i=1:ngcells
    conn = get(gcells(i), 'conn'); % connectivity
    x = gather(geom, conn, 'values', 'noreshape'); % coordinates of nodes
    us = gather(u, conn, 'values'); % displacements
    dTs = gather(dT, conn, 'values'); % temperatures
    Fe = zeros(dim*nfens,1); % element load vector
    % Loop over all integration points
    for j=1:npts_per_gcell
        N = Nmat (gcells(i), pc(j,:));
        Nder = Ndermat_param (gcells(i), pc(j,:));
        ts=Nder'*x; % rows: tangent vectors to parametric curves
        es=local_basis(self,ts); % calculate the local Cartesian basis
        xl=x*es(:,1:2);
        [Nspatialder,detF] = Ndermat_spatial (gcells(i), Nder, xl);
        B = Blmat (gcells(i), Nspatialder, es);
        D = tangent_moduli (mat, struct ('ms',matstates{i,j}));
        tepsilon = thermal_strains(mat,N'*dTs);
        Fe = Fe + B'*D* tepsilon * detF * thickness(gcells(i), pc(j,:)) * w(j);
    end
    evs(i) = set(evs(i), 'vec', Fe);
    evs(i) = set(evs(i), 'eqnums', gather(u, conn, 'eqnums'));
end
return;
